
#    MIDAS Httpd Server to provide Laser Data Acquisition Control Services
#    This controls the Data Acquisition driver, the program that controls
#    & collects data from the hardware.
#    It does not control the normal Data Acquisition program, which is done separately

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

source [file join $env(MIDASBASE) MIDASpackages LaserCmd1.0 pkgIndex.tcl]
package require LaserCmd

global MIDAS_LaserControl_Verbose
set MIDAS_LaserControl_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0


global MIDAS_DataAcquisitionCode
set MIDAS_DataAcquisitionCode 2

#    state returns

#    code (int)  state (string)  reason  (string)

#       0           stopped
#       1           going
#       2           initialised    "please setup"
#       3           setting up
#       4           changing to going
#       5           changing to stopped
#       6           error           ******
#       7           undefined       ******

global MIDAS_LaserRequest

set MIDAS_LaserRequest(0)   "READ_STATUS"
set MIDAS_LaserRequest(1)   "READ_DEBUG_STATUS"
set MIDAS_LaserRequest(3)   "READ_ACTIVE_SETTINGS"
set MIDAS_LaserRequest(4)   "READ_NEW_SETTINGS"
set MIDAS_LaserRequest(5)   "READ_DEBUG"
set MIDAS_LaserRequest(6)   "READ_NUM_CHANS"
set MIDAS_LaserRequest(7)   "READ_NUM_PULSES"
set MIDAS_LaserRequest(8)   "READ_DAC_STEP"
set MIDAS_LaserRequest(9)   "READ_DAC"
set MIDAS_LaserRequest(10)  "READ_RAMP_DAC"
set MIDAS_LaserRequest(11)  "READ_ENABLE_SCALER"
set MIDAS_LaserRequest(12)  "READ_ENABLE_ADC"
set MIDAS_LaserRequest(13)  "READ_ENABLE_TDC"
set MIDAS_LaserRequest(14)  "READ_DWELL_TIME"
set MIDAS_LaserRequest(15)  "READ_SETTLE_TIME"
set MIDAS_LaserRequest(16)  "READ_RUN_REQUEST"
set MIDAS_LaserRequest(17)  "READ_RUN_STATE"
set MIDAS_LaserRequest(18)  "READ_QUEUE_LEN"
set MIDAS_LaserRequest(19)  "READ_EVENT_COUNTER"
set MIDAS_LaserRequest(20)  "READ_CHANNEL"
set MIDAS_LaserRequest(21)  "READ_SCANS"
set MIDAS_LaserRequest(22)  "DEBUG"
set MIDAS_LaserRequest(23)  "NUM_CHANS"
set MIDAS_LaserRequest(24)  "NUM_PULSES"
set MIDAS_LaserRequest(25)  "DAC_STEP"
set MIDAS_LaserRequest(26)  "SET_DAC"
set MIDAS_LaserRequest(27)  "RAMP_DAC"
set MIDAS_LaserRequest(28)  "ENABLE_SCALER"
set MIDAS_LaserRequest(29)  "ENABLE_ADC"
set MIDAS_LaserRequest(30)  "ENABLE_TDC"
set MIDAS_LaserRequest(31)  "DWELL_TIME"
set MIDAS_LaserRequest(32)  "SETTLE_TIME"
set MIDAS_LaserRequest(33)  "RUN_REQUEST"
set MIDAS_LaserRequest(34)  "RUN_STATE"
set MIDAS_LaserRequest(35)  "RESET_QUEUE"
set MIDAS_LaserRequest(36)  "EVENT_COUNTER"
set MIDAS_LaserRequest(37)  "CHANNEL"

# Use namespaces to isolate our methods
namespace eval urn:LaserControl {
global MIDAS_LaserRequest

variable netargs ""
variable netvar 0
variable netz 0

proc first p {lindex $p 0}
proc last p {lindex $p [expr [llength $p]-1]}

    proc ServerVerbose {opt} {
        do_ServerVerbose $opt
        return [rpcvar::rpcvar int 0]
    }

    proc Laser-Request {req} {
        set vl [do_Request $req]
        return [rpcvar::rpcvar int $vl]
    }

#   define-register LASER DACstep	 "-d laser -p rw -r SS_DAC_STEP"
#   define-register LASER Rampit	 "-d laser -p rw -r SS_RAMP_DAC"
#   define-register LASER Settings.New "-d laser -p ro -r SS_NEW_SETTINGS"
#   define-register LASER Settings	 "-d laser -p ro -r SS_ACTIVE_SETTINGS"
#   define-register LASER Status	 "-d laser -p ro -r SS_STATUS"
#   define-register LASER Channel	 "-d laser -p rw -r SS_CHANNEL"
#   define-register LASER Event	 "-d laser -p rw -r SS_EVENT_COUNTER"
#   define-register LASER Scans	 "-d laser -p ro -r SS_SCANS"
#   define-register LASER Scaler	 "-d laser -p rw -r SS_ENABLE_SCALER -s1"
#   define-register LASER ADCEnable	 "-d laser -p rw -r SS_ENABLE_ADC"
#   define-register LASER TDCEnable	 "-d laser -p rw -r SS_ENABLE_TDC"

    proc Laser-GetRunState {} {
        set vl [do_Request "READ_RUN_STATE"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-RunRequest {r} {
        set vl [do_Request "RUN_REQUEST  $r"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-GetSettleTime {} {
        set vl [do_Request "READ_SETTLE_TIME"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-PutSettleTime {s} {
        set vl [do_Request "SETTLE_TIME $s"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-GetDwellTime {} {
        set vl [do_Request "READ_DWELL_TIME"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-PutDwellTime {s} {
        set vl [do_Request "DWELL_TIME $s"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-GetNumChans {} {
        set vl [do_Request "READ_NUM_CHANS"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-PutNumChans {c} {
        set vl [do_Request "NUM_CHANS $c"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-GetNumPulses {} {
        set vl [do_Request "READ_NUM_PULSES"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-PutNumPulses {p} {
        set vl [do_Request "NUM_PULSES $p"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-Start {} {
        set vl [do_Request "RUN_REQUEST RUN_START"]
        return [rpcvar::rpcvar int $vl]
    }

    proc Laser-Stop_Now {} {
        set vl [do_Request "RUN_REQUEST RUN_STOP_NOW"]
        return [rpcvar::rpcvar int $vl]
    }

# ------------------------------------------------------------------------

    proc do_ServerVerbose {option} {
        global MIDAS_LaserControl_Verbose
        if {$MIDAS_LaserControl_Verbose} {puts "called method ServerVerbose with parameter $option"}
        set MIDAS_LaserControl_Verbose $option
    }

    proc do_Request {args} {
       global env

       set env(EGsilent) 1
       set myargs [first $args]
       set z [catch {LaserControl $myargs} m]

       set env(EGsilent) 0

       if {$z == 0} then {
          return $m
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }



    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export Laser-Request
     SOAP::export Laser-Start 
     SOAP::export Laser-Stop_Now 
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /LaserControl -namespace urn:LaserControl -uri urn:LaserControl]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/LaserControl {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}
